cordova.define("cordova-plugin-ktplay.CordovaKTPlay", function(require, exports, module) {
/**
 * Cordova KTPlay Plugin
 * @class KTPLAY
 */
var exec = require('cordova/exec');
// var soundevent = new CustomEvent('handlesound');
// var communityevent = new CustomEvent('communitywindow');
// var messageevent = new CustomEvent('communitymessage');
// var logoutloginevent = new CustomEvent('logoutLogin');
// var interstitialevent = new CustomEvent('interstitial');
// var deeplinkevent = new CustomEvent('deeplink');


/**
 * This function is used to initialize the KTPlay SDK with app key and secret. SDK should be initialize first before making any other call.
 *
 * @method initSDK
 * @summary USAGE - window.plugins.ktplay.initSdk(options)
 * @param {Object} options - Object containing "key" and "secret" keys.
 * @example
 * var options = {
 *      key: 'myapikey', //string
 *      secret:'myappsecret' //string
 * };
 *
 * window.plugins.ktplay.initSdk(options);
 */
exports.initSdk = function (args) {
    exec(null, null, "CordovaKTPlay", "initKTPlaySdk", [args.key, args.secret]);
};


/**
 * This function is used to display the KTPlay default user login window
 *
 * @summary USAGE - window.plugins.ktplay.showLoginView(successCallback, errorCallback)
 * @param {function} successCallback - Callback function which will be called when login was successful with the details of user acount in Javascript Object.
 * @param {function} errorCallback - Callback function which will be called when their is some error in displaying the Login Window, error description will be send as a param to the function.
 * @example
 * var success = function(user){
 *    console.log("Login Successful", user);
 * };
 * var error = function(e){
 *    console.log("Community Window Display Error", e);
 * };
 * window.plugins.ktplay.showLoginView(sucess, error);
 * 
 * //Sample User Object
 *             user = {
 *                      autoGeneratedNickName: true
 *                      avatar: ""
 *                      city: ""
 *                      gameUserId: ""
 *                      gender: "Male"
 *                      leaderboardScore: "0"
 *                      loginType: ""
 *                      nickname: "ID448343509"
 *                      rank: "0"
 *                      snsUserId: ""
 *                      userID: "448343509"
 *              }
 * 
 */
exports.showLoginView = function (successCallback, errorCallback) {
    exec(successCallback, errorCallback, "CordovaKTPlay", "showLoginView", []);
};

/**
 * This function is used to login user with custom unique gamer id
 *
 * @summary USAGE - window.plugins.ktplay.customCommunityLogin(gamerid, successCallback, errorCallback)
 * @param {String} gamerid - Unique Gamer ID in String.
 * @param {function} successCallback - Callback function which will be called when login was successful with the details of user acount in Javascript Object.
 * @param {function} errorCallback - Callback function which will be called when their is some error in displaying the Login Window, error description will be send as a param to the function.
 * @example
 * var success = function(user){
 *    console.log("Login Successful", user);
 * };
 * var error = function(e){
 *    console.log("Community Window Display Error", e);
 * };
 * window.plugins.ktplay.showLoginView(sucess, error);
 * 
 * //Sample User Object
 *             user = {
 *                      autoGeneratedNickName: true
 *                      avatar: ""
 *                      city: ""
 *                      gameUserId: ""
 *                      gender: "Male"
 *                      leaderboardScore: "0"
 *                      loginType: ""
 *                      nickname: "ID448343509"
 *                      rank: "0"
 *                      snsUserId: ""
 *                      userID: "448343509"
 *              }
 * 
 */
exports.customCommunityLogin = function (gamerid, successCallback, errorCallback) {
    exec(successCallback, errorCallback, "CordovaKTPlay", "customCommunityLogin", [gamerid]);
    console.log(gamerid);
};

/**
 * This function is used to logout the currently logged in user. logoutlogin listener will be fired upon successful logout.
 *  @summary USAGE - window.plugins.ktplay.logoutUser();
 *  @example
 *  window.plugins.ktplay.logoutUser();
 */
exports.logoutUser = function () {
    exec(null, null, "CordovaKTPlay", "logoutUser", []);
};
/**
 * This function is used to get the User Profile based on the user id.
 *  @summary USAGE - window.plugins.ktplay.getUserProfile(userId, success, error);
 *  @param {String} userId - User id for which to fetch the details.
 *  @param {function} successCallback - Success Callback function which will be called with user data.
 *  @param {function} errorCallback - Error callback function which will be called when their is some error in retrieving the details.
 *  @example
 *  window.plugins.ktplay.getUserProfile('ID12312312312', success, error);
 */
exports.getUserProfile = function (userId, successCallback, errorCallback) {
    exec(successCallback, errorCallback, "CordovaKTPlay", "getUserProfile", [userId]);
};

/**
 * This function is used to update the User Profile of currently logged in user.
 *  @summary USAGE - window.plugins.ktplay.updateUserProfile(options, success, error);
 *  @param {Object} options - Javascript Object containing nickname, avatar and gender
 *  @param {function} successCallback - Success Callback function which will be called with new user data.
 *  @param {function} errorCallback - Error callback function which will be called when their is some error in updating the details.
 *  @example
 *  var options = {
 *      nickname: 'new nickname' //or null - if you do not want to update it
 *      avatarPath: 'file://avatar.png' //or null
 *      gender: 0 // 0 -No Change, 1 - Male, 2 - Female, 3 - Confidential (integer value)
 *  };
 *  window.plugins.ktplay.updateUserProfile(options, success, error);
 */
exports.updateUserProfile = function (args, successCallback, errorCallback) {
    exec(successCallback, errorCallback, "CordovaKTPlay", "updateUserProfile", [args.nickname, args.avatarPath, args.gender]);
};

/**
 *  This function is used to get the data of the currently logged in user.
 *  @summary USAGE - window.plugins.ktplay.getCurrentLoggedInUser(success, error);
 *  @param {function} successCallback - Success Callback function which will be called with the logged in user data.
 *  @param {function} errorCallback - Error callback function which will be called when their is some error in fetching the details.
 *  @example
 *  window.plugins.ktplay.getCurrentLoggedInUser(success, error);
 */
exports.getCurrentLoggedInUser = function (successCallback, errorCallback) {
    exec(successCallback, errorCallback, "CordovaKTPlay", "getCurrentLoggedInUser", []);
};

/**
 *  This function is used to get the data of the currently logged in user.
 *  @summary USAGE - window.plugins.ktplay.getCurrentLoggedInUser(success);
 *  @param {function} successCallback - Success Callback function which will be called with the logged in user data.
 *  @example
 *  var success = function(status){
 *      if(status == 1) {
 *          //user is logged in
 *      } else {
 *          //user is not logged in
 *      }
 *   };
 *   window.plugins.ktplay.getCurrentLoggedInUser(success);
 */
exports.getUserLoginStatus = function (successCallback) {
    exec(successCallback, null, "CordovaKTPlay", "getUserLoginStatus", []);
};

/**
 * This function is used to open a deeplink in KTPlay community window.
 *  @summary USAGE - window.plugins.ktplay.openDeeplink(deeplinkID, success);
 *  @param {String} deeplinkID - Deeplink ID.
 *  @param {function} successCallback - Success Callback function.
 *  @example
 *  window.plugins.ktplay.openDeeplink('ID12312312312', success);
 */
exports.openDeeplink = function (deeplinkID, successCallback) {
    exec(successCallback, null, "CordovaKTPlay", "openDeeplink", [deeplinkID]);
};

/**
 * This function is used to display the community window inside the app
 *
 * @summary USAGE - window.plugins.ktplay.showCommunityWindow(successCallback, errorCallback)
 * @param {function} successCallback - Callback function which will be called when the community window is displayed.
 * @param {function} errorCallback - Callback function which will be called when their is some error in displaying the community window, error description will be send as a param to the function.
 * @example
 * var success = function(){
 *   console.log("Community Window Displayed");
 * };
 * var error = function(e){
 *    console.log("Community Window Display Error", e);
 * };
 * window.plugins.ktplay.showCommunityWindow(sucess, error);
 * 
 */


exports.showCommunityWindow = function (successCallback, errorCallback) {
    exec(successCallback, errorCallback, "CordovaKTPlay", "showCommunityWindow", []);
};




/**
 * This function is used to hide/close the community window
 *
 * @summary USAGE - window.plugins.ktplay.hideCommunityWindow(successCallback, errorCallback)
 * @param {function} successCallback - Callback function which will be called when the community window is closed.
 * @param {function} errorCallback - Callback function which will be called when their is some error in closing the community window, error description will be send as a param to the function.
 * @example
 * var success = function(){
 *   console.log("Community Window Hidden");
 * };
 * var error = function(e){
 *    console.log("Community Window Hide Error", e);
 * };
 * window.plugins.ktplay.hideCommunityWindow(sucess, error);
 * 
 */
exports.hideCommunityWindow = function (successCallback, errorCallback) {
    exec(successCallback, errorCallback, "CordovaKTPlay", "hideCommunityWindow", []);
};

/**
 * This function is used to post Image or Video content to the community
 *
 * @summary USAGE - window.plugins.ktplay.postToCommunity(options, successCallback, errorCallback)
 * @param {Object} options - Javascript Object containing type, filepath, title and message. See Example below for details.
 * @param {function} successCallback - Callback function which will be called when the content is successfully posted.
 * @param {function} errorCallback - Callback function which will be called when their is some error in posting the content, error description will be send as a param to the function.
 * @example
 * var options = {
 *   type: 'image', // or 'video'
 *   filepath: 'file://myfile.png', //file location
 *   title: 'My Post Title',
 *   message: 'My Post Description'
 * };
 * var success = function(){
 *     console.log("Posted successfully);
 * };
 * var error = function(e){
 *    console.log("Post Error", e);
 * };
 * window.plugins.ktplay.postToCommunity(options, sucess, error);
 *
 */
exports.postToCommunity = function (args, successCallback, errorCallback) {
    exec(successCallback, errorCallback, "CordovaKTPlay", "postToCommunity", [args.type, args.filepath, args.title, args.message]);
};

/**
 * This function is used to update the KTPlay community language.
 *  @summary USAGE - window.plugins.ktplay.setCommunityLanguage(preferredLanguage, alternateLanguage, success, error);
 *  @param {String} preferredLanguage - Required Field. LanguageCode-LanguageScript-RegionCode format for e.g zh-CN. Language Code is mandatory field whereas Language Script and Region code is optional.
 *  @param {String} alternateLanguage - Optional Field in LanguageCode-LanguageScript-RegionCode format. Pass null if not setting the alternate language (not empty string).
 *  @param {function} successCallback - Success Callback function.
 *  @param {function} errorCallback - Error callback function.
 *  @example
 *
 *  window.plugins.ktplay.setCommunityLanguage("en-US", null, options, success, error);
 */
exports.setCommunityLanguage = function (preferredLanguage, alternateLanguage, successCallback, errorCallback) {
    exec(successCallback, errorCallback, "CordovaKTPlay", "setCommunityLanguage", [preferredLanguage, alternateLanguage]);
};

/**
 * This function is used to preload the Interstitial Message.
 *  @summary USAGE - window.plugins.ktplay.preloadInterstitialMessage(locationID, success, error);
 *  @param {String} locationId - Location ID or Trigger Point ID.
 *  @param {function} successCallback - Success Callback function.
 *  @param {function} errorCallback - Error callback function, error description will be send as a param to the function.
 *  @example
 *             window.plugins.ktplay.preloadInterstitialMessage('triggerid', success, error);
 */

exports.preloadInterstitialMessage = function (locationId, successCallback, errorCallback) {
    exec(successCallback, errorCallback, "CordovaKTPlay", "preloadInterstitialMessage", [locationId]);
};

/**
 * This function is used to show the Interstitial Message.
 *  @summary USAGE - window.plugins.ktplay.showInterstitialMessage(locationID, success, error);
 *  @param {String} locationId - Location ID or Trigger Point ID.
 *  @param {function} successCallback - Success Callback function.
 *  @example
 *  window.plugins.ktplay.showInterstitialMessage('triggerid', success);
 */
exports.showInterstitialMessage = function (locationId, successCallback) {
    exec(successCallback, null, "CordovaKTPlay", "showInterstitialMessage", [locationId]);
};


exports.executeCallback = function (task, status) {
    // console.log("Execute Callback", [task, status]);
    var event = document.createEvent('Event');
    switch (task) {
        /**
         * Listener is fired when the sound starts playing inside the community window. 'sound' key is added in the the data (object) returned by the listener, which is set to true when sound starts playing and false when its stop. See example for the detail.
         *
         * @event Sound
         * @param {Object} data -  data.sound - True if sound is playing, otherwise false.
         * @example
         *      function handlesound(data){
         *           if(data.sound){
         *              console.log("Sound Started");
         *           } else {
         *              console.log("Sound Stopped");
         *           }
         *      }
         *      document.addEventListener('handlesound', handlesound, false);
         */
        case 'sound':
            var data = (status == 1) ? true : false;
            event.initEvent('handlesound', true, true);
            event.sound = data;
            document.dispatchEvent(event);
            break;
            /**
             * Listener is fired when the community window is opened. 'communitywindow' key is added in the data (object) returned by the listener, which is set to true when the window is displayed and false when it is closed. See example for the detail.
             * @event Community Window
             * @param {Object} data - data.communitywindow - True if window is opened, otherwise false.
             * @example
             *      function comunityWindowStatus(data){
             *           if(data.communitywindow){
             *              console.log("Community Window Opened");
             *           } else {
             *              console.log("Community Window Closed");
             *           }
             *      }
             *      document.addEventListener('communitywindow', comunityWindowStatus, false);
             */
        case 'communitywindow':
            var data = (status == 1) ? true : false;
            // communityevent.communitywindow = data;
            event.initEvent('communitywindow', true, true);
            event.communitywindow = data;
            document.dispatchEvent(event);
            break;
            /**
             * Listener is fired when the community message is recieved. 'messagereceived' key is added in the data (object) returned by the listener, which is set to true when the message is recieved. See example for the detail.
             * @event Message
             * @param {Object} data - data.messagereceived - True if window is message is received.
             * @example
             *      function communitymessage(data){
             *           if(data.messagereceived){
             *              console.log("Community Message Received");
             *           } 
             *      }
             *      document.addEventListener('communitymessage', communitymessage, false);
             */
        case 'message':
            var data = (status == 1) ? true : false;
            // messageevent.messagereceived = data;
            event.initEvent('communitymessage', true, true);
            event.messagereceived = data;
            document.dispatchEvent(event);
            break;
            /**
             * Listener is fired when the user logs in or logs out. 'loggedin' and 'loggedout' key is added in the data (object) returned by the listener, which is set to true or false depending on the user action. See example for the detail.
             * @event Login/Logout
             * @param {Object} data - data.loggedin (True if user logs in) or data.loggedout (true if user logs out).
             * @example
             *      function logoutLogin(data){
             *           if(data.loggedin){
             *              console.log("User Logged In");
             *           }
             *           if(data.loggedout){
             *              console.log("User Logged Out");
             *           }
             *      }
             *      document.addEventListener('logoutLogin', logoutLogin, false);
             */
        case 'logoutlogin':
            event.initEvent('logoutLogin', true, true);
            if (status == 1) {
                event.loggedin = true;
                event.loggedout = false;
            } else {
                event.loggedin = false;
                event.loggedout = true;
            }
            document.dispatchEvent(event);
            break;
            /**
             * Listener is fired when the interstitial message is about to appear, cancelled or finished loading. See example for params.
             * @event Interstitital
             * @param {Object} data - data.willappear (True if message is about to appear) or data.cancelled (true if user cancel the interstitial message) or data.finsihed (True when it is finished loading).
             * @example
             *      function interstitial(data){
             *           if(data.willappear){
             *              console.log("Message will appear");
             *           }
             *           else if(data.cancelled){
             *              console.log("Message is cancelled");
             *           } else if(data.finsihed){
             *              console.log("Message is finsihed loading")
             *           }
             *      }
             *      document.addEventListener('interstitial', interstitial, false);
             */
        case 'interstitial':
            event.initEvent('interstitial', true, true);
            event.willappear = false;
            event.cancelled = false;
            event.finished = false;
            if (status == 'willappear') {
                event.willappear = true;
            } else if (status === 'cancelled') {
                event.cancelled = true;
            } else {
                event.finished = true;
            }
            document.dispatchEvent(event);
            break;
            /**
             * Listener is fired when user tap on a deeplink inside community window.
             * @event Deeplink
             * @param {Object} data - data.deeplink - deeplink schema url returned by KTPlay
             * @example
             *      function handledeeplink(data){
             *           console.log(data.deeplink);
             *      }
             *      document.addEventListener('deeplink', handledeeplink, false);
             */
        case 'deeplink':
            event.initEvent('deeplink', true, true);
            event.deeplink = status;
            document.dispatchEvent(event);
            break;
            /**
             * Listener is fired when the reward is recieved.
             * @event Rewards
             * @param {Object} data - data.rewards - Will contain a stringify object of rewards details. see example below for details. If there is an error, then data is 0
             * @example
             *      function rewardReceived(data){
             *           if(data != 0){
             *				var reward = JSON.parse(data);
             *				console.log(reward) // {ktUserId: "12312", messageId: "12312", gameUserId: "12312"}
             * 			 }
             *      }
             *      document.addEventListener('rewards', rewardReceived, false);
             */
        case 'rewards':
            event.initEvent('rewards', true, true);
            event.rewards = status;
            document.dispatchEvent(event);
            break;
        default:
            break;
    }
};
});
