cordova.define("cordova-plugin-tenjin.tenjin", function(require, exports, module) {
// Tenjin Cordova Plugin
// (c) 2017 Tenjin

var exec = require('cordova/exec');

/**
 * Manually track a transaction.
 *
 * @param {string} productId Name or ID of the product that you're selling
 * @param {string} currencyCode Currency code of the price, e.g. USD
 * @param {integer} quantity Number of transactions for this event
 * @param {double} unitPrice Unit price of a single transaction
 * @param {function} success Optional success callback
 * @param {function} failure Optional failure callback
 */
exports.transaction = function (productName, currency, quantity, unitPrice, success, failure) {

  if (typeof quantity !== 'number') {
    console.log('WARNING: expecting "quantity" to be an integer');
  }

  if (typeof unitPrice !== 'number') {
    console.log('WARNING: expecting "unitPrice" to be a floating point number');
  }

  exec(success, failure, 'Tenjin', 'transaction', [String(productName), String(currency), parseInt(quantity), parseFloat(unitPrice)]);
};

/**
 * Track an Android transaction and validate receipt data.
 *
 * @param {string} productId Name or ID of the product that you're selling
 * @param {string} currencyCode Currency code of the price, e.g. USD
 * @param {integer} quantity Number of transactions for this event
 * @param {double} unitPrice Unit price of a single transaction
 * @param {string} purchaseData JSON purchase data as a String
 * @param {string} dataSignature encoded signature
 * @param {function} success Optional success callback
 * @param {function} failure Optional failure callback
 */
exports.androidTransaction = function (productName, currency, quantity, unitPrice, purchaseData, dataSignature, success, failure) {

  if (typeof quantity !== 'number') {
    console.log('WARNING: expecting "quantity" to be an integer');
  }

  if (typeof unitPrice !== 'number') {
    console.log('WARNING: expecting "unitPrice" to be a floating point number');
  }

  exec(success, failure, 'Tenjin', 'androidTransaction',
    [String(productName), String(currency), parseInt(quantity), parseFloat(unitPrice), purchaseData, dataSignature]);
};

/**
 * Track an iOS transaction including the receipt data.
 *
 * @param {string} productId Name or ID of the product that you're selling
 * @param {string} currencyCode Currency code of the price, e.g. USD
 * @param {integer} quantity Number of transactions for this event
 * @param {double} unitPrice Unit price of a single transaction
 * @param {string} transactionId Transaction ID
 * @param {string} base64Receipt Receipt data as a base64 encoded string
 * @param {function} success Optional success callback
 * @param {function} failure Optional failure callback
 */
exports.iosTransaction = function (productName, currency, quantity, unitPrice, transactionId, base64Receipt, success, failure) {

  if (typeof quantity !== 'number') {
    console.log('WARNING: expecting "quantity" to be an integer');
  }

  if (typeof unitPrice !== 'number') {
    console.log('WARNING: expecting "unitPrice" to be a floating point number');
  }

  exec(success, failure, 'Tenjin', 'iosTransaction',
    [String(productName), String(currency), parseInt(quantity), parseFloat(unitPrice), transactionId, base64Receipt]);
};

/**
 * Send a custom event
 *
 * @param {string} eventName Custom event name
 * @param {function} success Optional success callback
 * @param {function} failure Optional failure callback
 */
exports.sendEvent = function (eventName, success, failure) {

  if (eventName === 'undefined') {
    console.log('WARNING: "eventName" is required');
    eventName = 'undefined';
  }

  exec(success, failure, 'Tenjin', 'sendEvent', [String(eventName)]);
};

/**
 * Send a custom event with an integer value
 *
 * @param {string} eventName Custom event name
 * @param {integer} integerValue Value to track with this metric
 * @param {function} success Optional success callback
 * @param {function} failure Optional failure callback
 */
exports.sendEventAndValue = function (eventName, integerValue, success, failure) {

  if (eventName === 'undefined') {
    console.log('WARNING: "eventName" is required');
    eventName = 'undefined';
  }

  if (typeof integerValue !== 'number') {
    console.log('WARNING: expecting "integerValue" to be an integer');
  }

  exec(success, failure, 'Tenjin', 'sendEventAndValue', [String(eventName), integerValue]);
};

});
